/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Epson_ESCP2_Instance.hpp"
#include "defines.hpp"

#include <iostream>
#include <strstream>

const static bool fDebugOutput       = false;
const static bool fTestNoCompression = true;

Epson_ESCP2_Instance::
Epson_ESCP2_Instance (Device *pDevice)
   : DeviceInstance (pDevice)
{
   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::Epson_ESCP2_Instance ()" << endl;

   fHaveSetupPrinter_d = false;
   ptlPrintHead_d.x    = 0;
   ptlPrintHead_d.y    = 0;
   pGamma_d            = 0;
   fHaveInitialized_d  = false;
}

Epson_ESCP2_Instance::
~Epson_ESCP2_Instance ()
{
   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::~Epson_ESCP2_Instance ()" << endl;

   if (pGamma_d)
   {
      delete pGamma_d;
      pGamma_d = 0;
   }
}

void Epson_ESCP2_Instance::
initializeInstance ()
{
   if (fHaveInitialized_d)
      return;

   fHaveInitialized_d = true;

   HardCopyCap      *pHCC = getCurrentForm ()->getHardCopyCap ();
   DeviceResolution *pDR  = getCurrentResolution ();
   DevicePrintMode  *pDPM = getCurrentPrintMode ();

   int iDitherID = getCurrentDitherID ();

   if (  pDPM->isID (DevicePrintMode::PRINT_MODE_24_COLOR)
      || pDPM->isID (DevicePrintMode::PRINT_MODE_24_BLACK_COLOR)
      || pDPM->isID (DevicePrintMode::PRINT_MODE_8_COLOR)
      || pDPM->isID (DevicePrintMode::PRINT_MODE_8_BLACK_COLOR)
      )
   {
      pGamma_d = getCurrentGamma ();

      // @TBD - why doesn't this work? if (fDebugOutput) cerr << dec << "pGamma_d = " << (pGamma_d ? *pGamma_d : "(null)") << endl;
      if (fDebugOutput)
      {
         if (pGamma_d)
            cerr << dec << "pGamma_d = " << *pGamma_d << endl;
         else
            cerr << dec << "pGamma_d = " << "(null)" << endl;
      }

      int iNumDstRowBytes8 = (pHCC->getXPels () + 7) >> 3;

      setDitherInstance (new GplDitherInstance (true,                        // fDataInRGB
                                                /* @TBD iBlackReduction*/0,  // iBlackReduction
                                                iDitherID,                   // iDitherType
                                                pDPM->getColorTech (),       // iColorTech
                                                pDR->getScanlineMultiple (), // iNumDitherRows
                                                pHCC->getXPels (),           // iSrcRowPels (was: iNumDstRowBytes8 * 8)
                                                iNumDstRowBytes8,            // iNumDestRowBytes
                                                pDR->getDstBitsPerPel (),    // iDestBitsPerPel
                                                pGamma_d));                  // pGamma
   }
}

bool Epson_ESCP2_Instance::
handleDeviceJobProperty (char *pszKey,
                         char *pszValue)
{
   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::handleDeviceJobProperty ("
                          << pszKey << ", "
                          << pszValue << ")"
                          << endl;

   return false;
}

void Epson_ESCP2_Instance::
setupPrinter ()
{
   if (fHaveSetupPrinter_d)
      return;

   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::setupPrinter ()" << endl;

   fHaveSetupPrinter_d = true;

   DeviceResolution *pRes      = getCurrentResolution ();
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   pCmd = pCommands->getCommandData ("cmdBeginRasterGraphics");
   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdBeginRasterGraphics defined for this device!" << endl;
   }

   pCmd = pCommands->getCommandData ("cmdSetResolution");
   if (pCmd)
   {
      pDevice_d->sendPrintfToDevice (pCmd, 3600 / pRes->getYRes ());
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdSetResolution defined for this device!" << endl;
   }

   DeviceForm *pForm = getCurrentForm ();

   pCmd = pCommands->getCommandData ("cmdSetFormSize");
   if (pCmd)
   {
      int iUnits = pForm->getHardCopyCap ()->getYPels ();
      if (fDebugOutput) cerr << dec << "iUnits = " << iUnits << endl;
      pDevice_d->sendPrintfToDevice (pCmd, iUnits);
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdSetFormSize defined for this device!" << endl;
   }

   // @TBD
   fUseMicroweave_d = false;

   if (pRes->isID (DeviceResolution::RESOLUTION_720_X_720))
   {
      fUseMicroweave_d = true;
   }

   pCmd = pCommands->getCommandData ("cmdBidi");
   if (pCmd)
   {
      char chState;

      if (fUseMicroweave_d)
         // Turn off bidirectional for microweaving
         chState = '0';
      else
         chState = '1';
      pDevice_d->sendPrintfToDevice (pCmd, chState);
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdBidi defined for this device!" << endl;
   }

   // @TBD
///pDevice_d->sendBinaryDataToDevice (getCurrentTray ());

   pCmd = pCommands->getCommandData ("cmdSetHardwareMicroweave");
   if (pCmd)
   {
      if (fUseMicroweave_d)
         pDevice_d->sendPrintfToDevice (pCmd, '1');
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdSetHardwareMicroweave defined for this device!" << endl;
   }

   // @TBD
   pCmd = pCommands->getCommandData ("cmdSetDotSize");
   if (pCmd)
   {
   }
   else
   {
      if (fDebugOutput) cerr << dec << "Error: There is no cmdSetDotSize defined for this device!" << endl;
   }
}

bool Epson_ESCP2_Instance::
beginJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::beginJob ()" << endl;

   pCmd = pCommands->getCommandData ("cmdUSBInit");

   if (  hasDeviceOption (USB_PORT_SUPPORT)
      && pCmd
      )
   {
      if (fDebugOutput) cerr << "cmdUSBInit = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdInit");
   if (pCmd)
   {
      if (fDebugOutput) cerr << "cmdInit = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool Epson_ESCP2_Instance::
newFrame ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::newFrame ()" << endl;

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
      if (fDebugOutput) cerr << "cmdPageEject = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   ditherNewFrame ();

   return true;
}

bool Epson_ESCP2_Instance::
newFrame (bool fJobPropertiesChanged)
{
   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::newFrame (with props)" << endl;
   if (fDebugOutput) cerr << "The job properties are as follows:" << endl;
   if (fDebugOutput) cerr << dec << "iOrientation = " << getCurrentOrientation() << endl;
   if (fDebugOutput) cerr << dec << "iDitherID = " << getCurrentDitherID () << endl;
   if (fDebugOutput) cerr << "The form is = " << getCurrentForm () << endl;
   if (fDebugOutput) cerr << "The tray is = " << getCurrentTray () << endl;
   if (fDebugOutput) cerr << "The media is = " << getCurrentMedia () << endl;
   if (fDebugOutput) cerr << "The resolution is = " << getCurrentResolution () << endl;
   if (fDebugOutput) cerr << "The print mode is = " << getCurrentPrintMode () << endl;

   // @TBD - reinitialize with new job properties

   // Call common code
   return newFrame ();
}

bool Epson_ESCP2_Instance::
endJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::endJob ()" << endl;

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
      if (fDebugOutput) cerr << "cmdPageEject = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdTerm");
   if (pCmd)
   {
      if (fDebugOutput) cerr << "cmdTerm = " << *pCmd << endl;

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   // @TBD - Pete hack!
   fHaveSetupPrinter_d = false;

   return true;
}

bool Epson_ESCP2_Instance::
abortJob ()
{
   if (fDebugOutput) cerr << "Epson_ESCP2_Instance::endJob ()" << endl;

   int cx = getCurrentForm ()->getHardCopyCap ()->getXPels ();

   // The largest size is 24 lines
   int iSize = 24 * (cx + 7) / 8;

   PBYTE abData = new BYTE [iSize];

   memset (abData, 0, sizeof (abData));

   BinaryData data (abData, sizeof (abData));

   pDevice_d->sendBinaryDataToDevice (&data);

   delete []abData;

   return true;
}

string Epson_ESCP2_Instance::
toString (ostrstream& oss)
{
   oss << hex
       << "{ "
       << DeviceInstance::toString (oss) // Add parent's output as well
       << " }" << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const Epson_ESCP2_Instance& const_self)
{
   Epson_ESCP2_Instance& self = const_cast<Epson_ESCP2_Instance&>(const_self);
   ostrstream            oss;

   os << self.toString (oss);

   return os;
}
